// Copyright (c) 2015, Mikhail Stolyarov. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

library do2config.base;

import 'dart:convert';

const String SECTION_DEFAULT = '_';

class Do2Config {

  final String name;
  
  Map<String,Map<String,dynamic>> get map => _param;
    
  String get json => JSON.encode(_param);
  
  Map<String,Map<String,dynamic>> _param = new Map();
  
  factory Do2Config.setJson(String name, String json){
    return new Do2Config.setMap(name, JSON.decode(json));
  }
  
  factory Do2Config.setMap(String name, Map<String,Map<String,dynamic>> config){
    Do2Config conf = new Do2Config(name);
    config.forEach((String section, Map<String,dynamic> param){
      param.forEach((String key, value){
        conf.set(key, value, section:section);
      });
    });
    return conf;
  }
  
  factory Do2Config.setIni(String name, String ini){
    Do2Config config = new Do2Config(name);
    String section = SECTION_DEFAULT;
    for(String line in ini.split('\n')){
      String paramLine = line.split(';')[0];
      if(paramLine.contains('=')){
        config.set(paramLine.split('=')[0].trim(), paramLine.split('=')[1].trim(), section:section);
      }
      else if(paramLine.contains('[') && paramLine.contains(']')){
        section = paramLine.split('[')[1].split(']')[0];
      }
    }
    return config;
  }

  Do2Config(String this.name);
  
  get(String nameParam, {dynamic defVal:null, String section:SECTION_DEFAULT}){
    if(_param[section] == null || _param[section][nameParam] == null){
      return defVal;
    }
    return _param[section][nameParam];
  }
  
  getInt(String nameParam,{int defVal:null, String section:SECTION_DEFAULT}) => int.parse(get(nameParam, defVal:defVal.toString(), section:section));
  
  getDouble(String nameParam,{double defVal:null, String section:SECTION_DEFAULT}) => double.parse(get(nameParam, defVal:defVal.toString(), section:section));
  
  getNum(String nameParam,{num defVal:null, String section:SECTION_DEFAULT}) => num.parse(get(nameParam, defVal:defVal.toString(), section:section));
  
  getString(String nameParam,{String defVal:null, String section:SECTION_DEFAULT}) => get(nameParam, defVal:defVal.toString(), section:section).toString();
  
  set(String nameParam, dynamic valueParam, {String section:SECTION_DEFAULT}){
    if(_param[section]== null){
      _param[section] = new Map();
    }
    _param[section][nameParam] = valueParam;
  }
  
  Map<String,dynamic> getSection([String section=SECTION_DEFAULT])=> _param[section];
  
  String getIni(){
    String outIni = ';$name \n\n';
    outIni += _section2String(SECTION_DEFAULT);
    _param.forEach((String section, Map<String, dynamic> param){
      if(section != SECTION_DEFAULT){
        outIni += '\n[$section]\n';
        outIni += _section2String(section);
      }

    });
    return outIni;
  }
  
  String _section2String(String section){
    String out='';
    getSection(section).forEach((String key, value){
      out += '$key = $value\n';
    });
    return out;
  }

}
