// Copyright (c) 2015, Mikhail Stolyarov. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

library do2config.test;

import 'package:test/test.dart';
import 'package:do2config/do2config.dart';

final String configIni = '''
key1 = var1
[header]
isConfigured = true ; boolean - true
sysConfig = RPIbp_X2at8
sensTotalCount = 8
sensMaxCount = 40
sysName = UM_1_RPI
sysSerial = 11223344
sysIP = 192.168.1.76
sysGW = 192.168.1.1
sysMask = 255.255.255.0
key2 = 1.23

[types]
sensTypesCount
type0 = Датчик температуры ;ds18b20
''';

main() {

  group('Do2Config', () {

    test('Create', () {
      Do2Config config = new Do2Config('test-Config');
      expect(config.name, equals('test-Config'));
      expect(config.get('param1',defVal:'default'), equals('default'));
      config.set('param1', 'test');
      expect(config.get('param1',defVal:'default'), equals('test'));
    });

    test('Create Json', (){
      Do2Config config = new Do2Config.setJson('TestName', '{"_":{"param1":"value1"}}');
      expect(config.name, equals('TestName'));
      expect(config.get('param1',defVal:'default'), equals('value1'));
      config.set('param1', 'test');
      expect(config.get('param1',defVal:'default'), equals('test'));
    });

    test('save to map', (){
      Do2Config config = new Do2Config.setJson('TestName', '{"_":{"param1":"value1"}}');
      expect(config.map, equals({'_':{'param1':'value1'}}));
    });

    test('save to json', (){
      Do2Config config = new Do2Config.setJson('TestName', '{"_":{"param1":"value1"}}');
      expect(config.json, equals('{"_":{"param1":"value1"}}'));
    });
    
    test('add ini config',(){
      Do2Config config = new Do2Config.setIni('test',configIni);
      expect(config.get('isConfigured',defVal:'default', section:'header'), equals('true'));
      expect(config.get('sysSerial',defVal:'default', section:'header'), equals('11223344'));
      expect(config.getSection('header'), equals({
        'isConfigured': 'true',
        'sysConfig': 'RPIbp_X2at8',
        'sensTotalCount': '8',
        'sensMaxCount': '40',
        'sysName': 'UM_1_RPI',
        'sysSerial': '11223344',
        'sysIP': '192.168.1.76',
        'sysGW': '192.168.1.1',
        'sysMask': '255.255.255.0',
        'key2': '1.23'
      }));
      expect(config.getSection(), equals({'key1': 'var1'}));
      expect(config.getSection('types'), equals({'type0': 'Датчик температуры'}));
      expect(config.getIni(), equals(';test \n'
          '\n'
          'key1 = var1\n'
          '\n'
          '[header]\n'
          'isConfigured = true\n'
          'sysConfig = RPIbp_X2at8\n'
          'sensTotalCount = 8\n'
          'sensMaxCount = 40\n'
          'sysName = UM_1_RPI\n'
          'sysSerial = 11223344\n'
          'sysIP = 192.168.1.76\n'
          'sysGW = 192.168.1.1\n'
          'sysMask = 255.255.255.0\n'
          'key2 = 1.23\n'
          '\n'
          '[types]\n'
          'type0 = Датчик температуры\n'
          ''));
    });
    
    test('get Int', (){
      Do2Config config = new Do2Config.setIni('test',configIni);
      expect(config.getInt('sensTotalCount',section:'header'), equals(8));
      expect(config.getInt('sensTotalCount1',defVal:1, section:'header'), equals(1));
    });
    
    test('get Double', (){
      Do2Config config = new Do2Config.setIni('test',configIni);
      expect(config.getDouble('key2',section:'header'), equals(1.23));
      expect(config.getDouble('key3', defVal:1.35, section:'header'), equals(1.35));
    });
    
    test('get Num', (){
      Do2Config config = new Do2Config.setIni('test',configIni);
      expect(config.getNum('key2',section:'header'), equals(1.23));
      expect(config.getNum('key3', defVal:1.35, section:'header'), equals(1.35));
    });
    
    test('get String', (){
      Do2Config config = new Do2Config.setIni('test',configIni);
      expect(config.getString('key2',section:'header'), equals('1.23'));
      expect(config.getString('key3', defVal:'1.35', section:'header'), equals('1.35'));
    });
    
  });

}
